@@ -10,6 +10,8 @@ contains the bits of INTEGER."
 (defun %rotate-byte (count size pos integer)
   (let ((count (nth-value 1 (round count size)))
         (mask (1- (ash 1 size))))
+    (when (>= pos (integer-length integer))
+      (return-from %rotate-byte integer))
     (logior (logand integer (lognot (ash mask pos)))
             (let ((field (logand (ash mask pos) integer)))
               (logand (ash mask pos)
@@ -24,6 +26,6 @@ contains the bits of INTEGER."
   (declare (notinline %rotate-byte))
   (%rotate-byte count 32 0 integer))
 
-#+x86-64
+#+64-bit
 (defun %unsigned-64-rotate-byte (count integer)
   (%unsigned-64-rotate-byte count integer))
