@@ -9,25 +9,25 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
-(define-vop (debug-cur-sp)
-  (:translate sb!di::current-sp)
+(define-vop ()
+  (:translate current-sp)
   (:policy :fast-safe)
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
   (:generator 1
     (move res csp-tn)))
 
-(define-vop (debug-cur-fp)
-  (:translate sb!di::current-fp)
+(define-vop (current-fp-sap)
+  (:translate current-fp)
   (:policy :fast-safe)
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
   (:generator 1
     (move res cfp-tn)))
 
-(define-vop (read-control-stack)
+(define-vop ()
   (:translate stack-ref)
   (:policy :fast-safe)
   (:args (sap :scs (sap-reg))
@@ -38,18 +38,15 @@
   (:generator 5
     (inst lwzx result sap offset)))
 
-(define-vop (write-control-stack)
+(define-vop ()
   (:translate %set-stack-ref)
   (:policy :fast-safe)
   (:args (sap :scs (sap-reg))
          (offset :scs (any-reg))
-         (value :scs (descriptor-reg) :target result))
+         (value :scs (descriptor-reg)))
   (:arg-types system-area-pointer positive-fixnum *)
-  (:results (result :scs (descriptor-reg)))
-  (:result-types *)
   (:generator 5
-    (inst stwx value sap offset)
-    (move result value)))
+    (inst stwx value sap offset)))
 
 (define-vop (code-from-mumble)
   (:policy :fast-safe)
@@ -63,23 +60,23 @@
       (loadw temp thing 0 lowtag)
       (inst srwi temp temp n-widetag-bits)
       (inst cmpwi temp 0)
-      (inst slwi temp temp (1- (integer-length n-word-bytes)))
+      (inst slwi temp temp word-shift)
       (inst beq bogus)
       (unless (= lowtag other-pointer-lowtag)
         (inst addi temp temp (- lowtag other-pointer-lowtag)))
       (inst sub code thing temp)
       (emit-label done)
-      (assemble (*elsewhere*)
+      (assemble (:elsewhere)
         (emit-label bogus)
         (move code null-tn)
         (inst b done)))))
 
 (define-vop (code-from-lra code-from-mumble)
-  (:translate sb!di::lra-code-header)
+  (:translate sb-di::lra-code-header)
   (:variant other-pointer-lowtag))
 
 (define-vop (code-from-fun code-from-mumble)
-  (:translate sb!di::fun-code-header)
+  (:translate sb-di::fun-code-header)
   (:variant fun-pointer-lowtag))
 
 (define-vop (%make-lisp-obj)
@@ -93,20 +90,9 @@
 
 (define-vop (get-lisp-obj-address)
   (:policy :fast-safe)
-  (:translate sb!di::get-lisp-obj-address)
-  (:args (thing :scs (descriptor-reg) :target result))
+  (:translate sb-di::get-lisp-obj-address)
+  (:args (thing :scs (descriptor-reg any-reg) :target result))
   (:results (result :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 1
     (move result thing)))
-
-
-(define-vop (fun-word-offset)
-  (:policy :fast-safe)
-  (:translate sb!di::fun-word-offset)
-  (:args (fun :scs (descriptor-reg)))
-  (:results (res :scs (unsigned-reg)))
-  (:result-types positive-fixnum)
-  (:generator 5
-    (loadw res fun 0 fun-pointer-lowtag)
-    (inst srwi res res n-widetag-bits)))
