@@ -549,7 +549,8 @@ void sb_nanosleep_float(float seconds) {
 }
 #endif
 
-#ifdef LISP_FEATURE_NETBSD
+/* 64-bit timeval */
+#if !defined(LISP_FEATURE_64_BIT) || defined (LISP_FEATURE_NETBSD)
 /* These thin wrappers are needed due to "linker rewriting"
  * acording to git revision 9304704f68 */
 int sb_getrusage(int who, struct rusage *rusage)
@@ -615,7 +616,9 @@ SB_TRIG_WRAPPER(atanh)
 double sb_hypot (double x, double y) {
     return hypot(x, y);
 }
-
+double sb_hypotf (double x, double y) {
+    return hypotf(x, y);
+}
 #endif
 
 int sb_fileno(FILE* f) { return fileno(f); } // might be a C macro
