@@ -770,6 +770,7 @@
    (lambda (x y)
      (ldb (byte y (- 32 y)) x))
    (unsigned-byte 32))
+  #+64-bit
   (assert (not (ctu:ir1-named-calls `(lambda (x y)
                                        (ldb (byte y (- 32 y)) x))))))
 
@@ -792,7 +793,13 @@
    `(lambda (c)
       (logand (+ c (multiple-value-prog1 0)) 1))
    ((1) 1)
-   ((2) 0)))
+   ((2) 0))
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (oddp (1+ (read-from-string a))))
+   (("1") nil)
+   (("-2") t)))
 
 (with-test (:name :logtest-lognot)
   (checked-compile-and-assert
@@ -801,3 +808,214 @@
       (logtest -3 (lognot a)))
    ((5) t)
    ((-3) nil)))
+
+(with-test (:name :transform-deleted-casts)
+  (checked-compile-and-assert
+   (:optimize :safe)
+   `(lambda (x y)
+      (declare ((integer 0 30) y))
+      (dpb 3 (byte 2 y) (the (integer * 300) x)))
+   ((400 2) (condition 'type-error))
+   ((300 0) 303)))
+
+(with-test (:name :truncate-unknown-integer)
+  (assert (not (ctu:ir1-named-calls `(lambda (x)
+                                       (declare (integer x))
+                                       (values (the (signed-byte 25) (floor x 2)))))))
+  (assert (not (ctu:ir1-named-calls `(lambda (x)
+                                       (declare (unsigned-byte x))
+                                       (values (the (signed-byte 25) (floor x 2)))))))
+  (assert (not (ctu:ir1-named-calls `(lambda (x)
+                                       (declare (integer x))
+                                       (values (the (signed-byte 25) (truncate x 2)))))))
+  (assert (not (ctu:ir1-named-calls `(lambda (x)
+                                       (declare (unsigned-byte x))
+                                       (values (the (signed-byte 25) (truncate x 2)))))))
+  (assert (not (ctu:ir1-named-calls `(lambda (x)
+                                       (declare (integer x))
+                                       (values (the (signed-byte 25) (ceiling x 2)))))))
+  (assert (not (ctu:ir1-named-calls `(lambda (x)
+                                       (declare (unsigned-byte x))
+                                       (values (the (signed-byte 25) (ceiling x 2))))))))
+
+(with-test (:name :word-floor-ceiling)
+  (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                       (declare (sb-vm:signed-word x y))
+                                       (floor x y)))))
+  (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                       (declare (sb-vm:signed-word x y))
+                                       (ceiling x y)))))
+  (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                       (declare (sb-vm:word x y))
+                                       (ceiling x y)))))
+  (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                       (declare (sb-vm:word x y))
+                                       (floor x y))))))
+
+(with-test (:name :fixnum-*-by-unknown)
+  (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                       (declare (integer x)
+                                                (fixnum y))
+                                       (the fixnum (* x y))))))
+  (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                       (declare (integer x)
+                                                (fixnum y))
+                                       (the fixnum (* y x))))))
+  (assert (equal (ctu:ir1-named-calls `(lambda (x)
+                                         (the fixnum (* x 2))))
+                 '(sb-kernel:*-by-fixnum-to-fixnum)))
+  (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                       (declare (integer x y))
+                                       (the fixnum (* x y))))))
+  (checked-compile-and-assert
+      (:optimize :safe)
+      `(lambda (a)
+        (declare ((integer -1 11094097273866491717) a))
+        (the (integer 1)
+             (truncate a 2305843009213693949)))
+    ((11094097273866491717) (values 4 1870725237011715921))))
+
+(with-test (:name :logxor-1-type)
+  (assert-type
+   (lambda (x)
+     (logxor x (1- x)))
+   (integer -1))
+  (assert-type
+   (lambda (x)
+     (declare ((integer 1) x))
+     (logxor x (1- x)))
+   (integer 1))
+  (assert-type
+   (lambda (x)
+     (logxor x (1+ x)))
+   (integer -1))
+  (assert-type
+   (lambda (x)
+     (unless (eql x -1)
+      (logxor x (1+ x))))
+   (or null (integer 1))))
+
+(with-test (:name :unsigned-byte-x-p)
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (typep a '(unsigned-byte 128)))
+   ((1) t)
+   ((-1) nil)
+   (((1- (expt 2 128))) t)
+   (((expt 2 128)) nil)
+   (((- (expt 2 128))) nil)
+   (('a) nil))
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (declare (unsigned-byte a))
+      (typep a '(unsigned-byte 128)))
+   ((1) t)
+   (((1- (expt 2 128))) t)
+   (((expt 2 128)) nil))
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (declare (integer a))
+      (typep a '(unsigned-byte 128)))
+   ((1) t)
+   ((-1) nil)
+   (((1- (expt 2 128))) t)
+   (((expt 2 128)) nil)
+   (((- (expt 2 128))) nil))
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (declare (bignum a))
+      (typep a '(unsigned-byte 128)))
+   (((1- (expt 2 128))) t)
+   (((expt 2 128)) nil)
+   (((- (expt 2 128))) nil))
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (typep a '(unsigned-byte 80)))
+   ((1) t)
+   ((-1) nil)
+   (((1- (expt 2 80))) t)
+   (((expt 2 80)) nil)
+   (((- (expt 2 80))) nil)
+   (('a) nil))
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (typep a '(unsigned-byte 160)))
+   ((1) t)
+   ((-1) nil)
+   (((expt 2 80)) t)
+   (((1- (expt 2 160))) t)
+   (((expt 2 160)) nil)
+   (((- (expt 2 160))) nil)
+   (('a) nil)))
+
+(with-test (:name :signed-byte-x-p)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a)
+         (typep a '(signed-byte 128)))
+    ((1) t)
+    ((-2) t)
+    (((1- (expt 2 127))) t)
+    (((- (expt 2 127))) t)
+    (((expt 2 127)) nil)
+    (('a) nil))
+  (checked-compile-and-assert
+      ()
+      `(lambda (a)
+         (declare (integer a))
+         (typep a '(signed-byte 128)))
+    ((1) t)
+    ((-2) t)
+    (((1- (expt 2 127))) t)
+    (((- (expt 2 127))) t)
+    (((expt 2 127)) nil))
+  (checked-compile-and-assert
+      ()
+      `(lambda (a)
+         (declare (bignum a))
+         (typep a '(signed-byte 128)))
+    (((1- (expt 2 127))) t)
+    (((- (expt 2 127))) t)
+    (((expt 2 127)) nil)))
+
+(with-test (:name :%bignum-length-type)
+  (assert-type
+   (lambda (x)
+     (declare (sb-vm:signed-word x))
+     (sb-bignum:%bignum-length x))
+   (integer 1 1))
+  (assert-type
+   (lambda (x)
+     (declare (word x))
+     (sb-bignum:%bignum-length x))
+   (integer 1 2)))
+
+(with-test (:name :minus-eql-to-eq-reduction)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a c)
+         (declare (integer c))
+         (when (integerp a)
+           (eql (- a c) 0)))
+      (((eval '(expt 2 64)) (eval '(expt 2 64))) t)
+      ((1 2) nil)))
+
+(with-test (:name :unsigned-byte-64-p-move-to-word)
+  (checked-compile-and-assert
+   ()
+   `(lambda (a m)
+      (if (typep a '(unsigned-byte 64))
+          (let ((j a))
+            (declare ((unsigned-byte 64) j))
+            (loop repeat 2
+                  sum
+                  (+ (funcall m (logand j #xFFFFFF))
+                     (funcall m (logand j #xFFFFFF)))))))
+   ((1234 #'+) 4936)
+   ((3 #'-) -12)))
