@@ -329,28 +329,28 @@
                             (f)))))
                      :key (lambda (x) (combination-fun-source-name x nil))))))
 
+(defun count-type-checks (lambda)
+  (count-if (lambda (name)
+              (member name '(sb-c::%type-check-error/c sb-c::%type-check-error)))
+         (ir-calls lambda)
+         :key (lambda (x) (combination-fun-source-name x nil))))
+
 (with-test (:name :instance-constraint-intersection)
-  (assert (not (find 'sb-c::%type-check-error/c
-                     (ir-calls
-                      `(lambda (x)
-                         (typecase x
-                           (stream 2)
-                           (hash-table 1))))
-                     :key (lambda (x) (combination-fun-source-name x nil))))))
+  (assert (zerop (count-type-checks
+                  `(lambda (x)
+                     (typecase x
+                       (stream 2)
+                       (hash-table 1)))))))
 
 (with-test (:name :aref-full-call-no-type-check)
-  (assert (not (find 'sb-c::%type-check-error/c
-                     (ir-calls
-                      `(lambda (x)
-                         (aref x 0)))
-                     :key (lambda (x) (combination-fun-source-name x nil))))))
+  (assert (zerop (count-type-checks
+                  `(lambda (x)
+                     (aref x 0))))))
 
 (with-test (:name :call-full-like-p-constants)
-  (assert (not (find 'sb-c::%type-check-error/c
-                     (ir-calls
-                      `(lambda (a b)
-                         (< (truly-the double-float a) b)))
-                     :key (lambda (x) (combination-fun-source-name x nil))))))
+  (assert (zerop (count-type-checks
+                  `(lambda (a b)
+                     (< (truly-the double-float a) b))))))
 
 (with-test (:name :constant-substitution)
   (let ((calls (ir-calls
@@ -421,21 +421,17 @@
       0)))
 
 (with-test (:name :let-no-typecheck)
-  (assert (not (find 'sb-c::%type-check-error/c
-                     (ir-calls
-                      `(lambda (x)
-                         (let ((m (the sequence x)))
-                           (values (length m)
-                                   m))))
-                     :key (lambda (x) (combination-fun-source-name x nil)))))
-  (assert (eql (count 'sb-c::%type-check-error/c
-                      (ir-calls
-                       `(lambda (x l)
-                          (let ((m (the sequence x))
-                                (l (the integer l)))
-                            (values (length m)
-                                    l))))
-                      :key (lambda (x) (combination-fun-source-name x nil)))
+  (assert (zerop (count-type-checks
+                  `(lambda (x)
+                     (let ((m (the sequence x)))
+                       (values (length m)
+                               m))))))
+  (assert (eql (count-type-checks
+                `(lambda (x l)
+                   (let ((m (the sequence x))
+                         (l (the integer l)))
+                     (values (length m)
+                             l))))
                1)))
 
 (with-test (:name :pop-special-once)
@@ -446,8 +442,8 @@
                          (pop s))))
       1)))
 
-(with-test (:name :overflow+make-array
-                  :fails-on (not :64-bit))
+#+(or x86-64 arm64)
+(with-test (:name :overflow+make-array)
   (assert
    (= (count 'sb-vm::overflow+t
              (ir2-vops '(lambda (y)
@@ -455,29 +451,32 @@
       1)))
 
 (with-test (:name :other-pointer-p)
-  (assert (not (find 'sb-c::%type-check-error/c
-                     (ir-calls
-                      `(lambda (x)
-                         (when (and (stringp (truly-the (or simple-string (member #\a)) x))
-                                    (zerop (length x)))
-                           x)))
-                     :key (lambda (x) (combination-fun-source-name x nil))))))
+  (assert (zerop (count-type-checks
+                  `(lambda (x)
+                     (when (and (stringp (truly-the (or simple-string (member #\a)) x))
+                                (zerop (length x)))
+                       x))))))
 
 (with-test (:name :external-type-checks-across-functions)
-  (assert (not (find 'sb-c::%type-check-error/c
-                     (ir-calls
-                      `(lambda (a b)
-                         (declare (number a b)
-                                  (optimize speed))
-                         (+ a b)))
-                     :key (lambda (x) (combination-fun-source-name x nil))))))
+  (assert (zerop (count-type-checks
+                  `(lambda (a b)
+                     (declare (number a b)
+                              (optimize speed))
+                     (+ a b))))))
 
 (with-test (:name :consecutive-casts)
-  (assert (= (count 'sb-c::%type-check-error/c
-                    (ir-calls
-                     `(lambda (x)
-                        (the fixnum (the integer x))))
-                    :key (lambda (x) (combination-fun-source-name x nil)))
+  (assert (= (count-type-checks
+              `(lambda (x)
+                 (the fixnum (the integer x))))
+             1))
+  #+(or arm64 x86-64)
+  (assert (= (count-type-checks
+              `(lambda (x)
+                 (logand (the number x) 2)))
+             0))
+  (assert (= (count-type-checks
+              `(lambda (x)
+                 (the integer (the (real 5) x))))
              1)))
 
 (with-test (:name :sign-extend)
@@ -504,20 +503,110 @@
                     :key (lambda (x) (combination-fun-source-name x nil)))
              1)))
 
-
 (with-test (:name :optional-type-checks)
-  (assert (= (count 'sb-c::%type-check-error/c
-                    (ir-calls
-                     `(lambda (&optional x y)
-                        (declare (list x))
-                        (values x y)))
-                    :key (lambda (x) (combination-fun-source-name x nil)))
+  (assert (= (count-type-checks
+              `(lambda (&optional x y)
+                 (declare (list x))
+                 (values x y)))
              1)))
 
-
 (with-test (:name :flush-multiple-callables)
   (assert (not (ir-full-calls
                 `(lambda (a b c)
                    (declare (vector b))
                    (find a b :test (if c #'eq #'eql))
                    10)))))
+
+(with-test (:name :stack-allocate-make-array-reverse)
+  (let ((vops (ir2-vops `(lambda (l)
+                           (let ((j (make-array 5 :initial-contents (nreverse l))))
+                             (declare (dynamic-extent j))
+                             (opaque-identity j)
+                             10)))))
+    (assert (= (count 'sb-vm::allocate-vector-on-stack vops) 1))
+    (assert (= (count 'sb-vm::allocate-vector-on-heap vops) 0))))
+
+
+(with-test (:name :no-type-check-tail-call)
+  (destructuring-bind (combination)
+      (ir-full-calls `(lambda (x)
+                        (truly-the fixnum (funcall (the function x)))))
+    (assert (node-tail-p combination))))
+
+(with-test (:name :evenp+arithmetic)
+  (assert (not (ir-full-calls `(lambda (x)
+                                 (evenp (+ x 3))))))
+  (assert (not (ir-full-calls `(lambda (x)
+                                 (logbitp 0 (+ x 3)))))))
+
+(with-test (:name :modarith-unknown-types)
+  (assert (not (ir-full-calls `(lambda (x)
+                                 (logand (+ x 10) 20)))))
+  (assert (not (ir-full-calls `(lambda (m x)
+                                 (logand (if m (+ x 3) (- x 2)) 20))))))
+
+(with-test (:name :reoptimize-complement)
+  (assert (not (ir-full-calls `(lambda (x)
+                                 (declare ((simple-array fixnum (*)) x)
+                                          (optimize speed))
+                                 (position 1 x :test-not #'=))))))
+
+(with-test (:name :complement-multiple-calls)
+  (assert (= (count 'complement
+                    (ir-calls
+                     `(lambda (m n)
+                        (let ((c (complement #'=)))
+                          (values (funcall c 1 m)
+                                  (funcall c 3 n)))))
+                    :key (lambda (x) (combination-fun-source-name x nil)))
+             0))
+  (assert (= (count 'complement
+                    (ir-calls
+                     `(lambda (l)
+                        (declare (list l)
+                                 (optimize speed))
+                        (position 10 l :test (complement #'=))))
+                    :key (lambda (x) (combination-fun-source-name x nil)))
+             0)))
+
+(with-test (:name :local-calls-to-&rest)
+  (assert (not (ir-full-calls
+                `(lambda (a)
+                   (flet ((a (&rest args)
+                            (apply #'eql args)))
+                     (list (a a 1)
+                           (a a 2))))))))
+
+(with-test (:name :inline-local-call-with-casts)
+  (assert (not (ir-full-calls
+                `(lambda (n)
+                   (funcall (the (function (fixnum) fixnum)
+                                 (lambda (x) (1+ x)))
+                            n))))))
+
+(with-test (:name :truncate-signed-word-error)
+  (assert (not (find 'sb-vm::move-from-signed
+                     (ir2-vops '(lambda (x d)
+                                 (declare ((signed-byte 64) x d))
+                                 (values (the fixnum (truncate x d)))))))))
+
+(with-test (:name :cast-movement)
+  (assert (not (find 'sb-vm::move-from-unsigned
+                     (ir2-vops '(lambda (d x c)
+                                 (declare ((simple-array word (4)) x))
+                                 (let* ((b (aref x 1))
+                                        (a (if d c b)))
+                                   (> (the word a) 10)))))))
+  (assert (find 'sb-vm::move-from-unsigned
+                (ir2-vops '(lambda (d x c)
+                            (declare ((simple-array word (4)) x))
+                            (let* ((b (aref x 1))
+                                   (a (if d c b)))
+                              (print 1)
+                              (> (the word a) 10)))))))
+
+(with-test (:name :overflow-svref
+            :skipped-on (not (or :arm64 :x86-64)))
+  (assert (not (ir-full-calls
+                `(lambda (x n)
+                   (svref x (+ n 1)))))))
