@@ -1,4 +1,19 @@
 
+(defun adjacent-barriers ()
+  (let ((o (list 1)))
+    (setf (car o) (list 100))
+    (setf (car o) (make-array 200 :element-type 'fixnum))
+    (setf (cdr o) (list 100 2))))
+
+(with-test (:name :adjacent-barrier-elimination-sticky-marks
+            :skipped-on (or :gc-stress
+                            (:not :soft-card-marks)))
+  (setf (extern-alien "pre_verify_gen_0" int) 1)
+  (loop repeat 1000000
+        do (adjacent-barriers))
+  (setf (extern-alien "pre_verify_gen_0" int) 0))
+
+
 ;;; The layout of FOO must not get promoted from gen0 to gen1.
 ;;; Due to random variation, there might be a gc-with-promotion cycle
 ;;; just after the defstruct. So prevent that.
@@ -11,7 +26,8 @@
 
 (with-test (:name :compact-instance-header-layout
             :skipped-on (or :win32
-                            (:not (:and :compact-instance-header :soft-card-marks)))
+                            :gc-stress
+                            (:not (:and :gencgc :compact-instance-header :soft-card-marks)))
             :fails-on (and :darwin :x86-64)) ;; can't compile the .so
   (unless (probe-file "gc-testlib.so")
     (sb-ext:run-program "sh"
