@@ -26,7 +26,7 @@
 
 (let ((*evaluator-mode* :compile))
   (with-compilation-unit () (load"run-tests")))
-#+(and x86-64 linux sb-thread)
+#+(and x86-64 linux sb-thread (not sb-safepoint))
   (unless (or (find :mark-region-gc sb-impl:+internal-features+)
               ;; %LISTIFY-REST-ARGS has a problem with profiling because it uses
               ;; the JRCXZ instruction, which can only take an 8-bit signed disp.
@@ -38,7 +38,7 @@
 (in-package run-tests)
 (import '(sb-alien:alien-funcall sb-alien:extern-alien
           sb-alien:int sb-alien:c-string sb-alien:unsigned))
-(setq *summarize-test-times* t)
+(setq *summarize-test-times* (not (sb-ext:posix-getenv "SBCL_TEST_NO_SUMMARIZE")))
 ;;; Ordered approximately in descending order by running time
 (defvar *timings* (with-open-file (s "timing") (read s)))
 
@@ -127,6 +127,9 @@
                          (when (or (search ".pure" filename) (search ".impure" filename))
                            (push filename missing-usage))))
                      (cond ((eq code 104)
+                            (when (and *delete-logs*
+                                       (equal (pathname-type filename) "test"))
+                              (delete-file (format nil "~a/~a~@[-~d~]" *logdir* filename iteration)))
                             (format t "~A: success (~d msec)~%" filename et))
                            (t
                             (format t "~A~@[[~d]~]: status ~D (~d msec)~%"
@@ -194,7 +197,11 @@
                    (when (member :allocator-metrics sb-impl:+internal-features+)
                      (format t "~2&Allocator histogram:~%")
                      (funcall (intern "PRINT-ALLOCATOR-HISTOGRAM" "SB-THREAD")))
-                   #+test-sprof (sb-sprof:report :type :flat)
+                   #+test-sprof
+                   (sb-sprof:report :type :flat
+                                    :stream (if (sb-ext:posix-getenv "SBCL_TEST_NO_SUMMARIZE")
+                                                (make-broadcast-stream)
+                                                *standard-output*))
                    #+tlsf-stress (cl-user::tlsf-dump)
                    #-arm64 ;; causes crashes
                    (gc :gen 7)
@@ -231,6 +238,10 @@
 (when (string= (car *posix-argv*) "--filter")
   (setq *filter* (cadr *posix-argv*))
   (setq *posix-argv* (cddr *posix-argv*)))
+(when (find "--delete-logs" *posix-argv* :test #'equal)
+  (setf *delete-logs* t)
+  (setf *posix-argv* (remove "--delete-logs" *posix-argv* :test #'equal)))
+
 (if (<= (length *posix-argv*) 1)
     ;; short form - test all files. Argument N if specified is the number of
     ;; tasks, defaulting to half the machine's reported cores
@@ -254,8 +265,10 @@
                                 (impure-cload-files)
                                 (sh-files)))))
        jobs
-       t)
-      #+(and linux sb-thread 64-bit) (summarize-gc-times))
+       (not (sb-ext:posix-getenv "SBCL_TEST_NO_SUMMARIZE")))
+      #+(and linux sb-thread 64-bit)
+      (unless (sb-ext:posix-getenv "SBCL_TEST_NO_SUMMARIZE")
+        (summarize-gc-times)))
     ;; long form
     (let ((jobs 4)
           (runs-per-test 1)
